// [ToDo] 
// - bug when resizing the window while a tree is maximized

#include "View.h"

#include <hsm/svgl/SVGLWindow.h>
#include <hsm/svgl/Svgl.h>
#include <hsm/component/Point.h>
#include <hsm/util/Debug.h>

#include <w3c/svg/SVGGElement.hpp>
#include <w3c/svg/SVGRectElement.hpp>
#include <w3c/svg/SVGTextElement.hpp>
#include <w3c/svg/svgUseElement.hpp>

#include <w3c/svg/SVGTransforms.hpp>
#include <w3c/svg/SVGTransformable.hpp>


View::View(const std::string &label, hsm::SVGLWindow *w, CompositeView *p /* = 0 */) : hsm::impl::Component<>(label), window(w), parent(p) {
	view = new svg::SVGGElement(window->svgDocument);
	view->setFontSize(13);
	view->setFontFamily(unicode::String::createString("Andale Mono"));
	view->setStroke("gray");
	
	if(parent == 0) {
		(*window/"size").addReceiver(this);
	}
	
	viewBg = new svg::SVGRectElement(window->svgDocument);
	viewBg->setFill("url(#bgd)");
	view->appendChild(viewBg);
	
	name = new svg::SVGTextElement(window->svgDocument);
	name->setText(label.c_str());
	name->setY(12);
	name->rotate(-90);
	view->appendChild(name);
	
	svg::SVGUseElement *close = new svg::SVGUseElement(window->svgDocument);
	close->setHref("#close");
	close->setX(8); close->setY(9);
	
	resize = new svg::SVGUseElement(window->svgDocument);
	resize->setHref("#max");
	resize->setX(8); resize->setY(22);
	hsm::svgl::setAttribute(resize, "hsm:behaviour", "viewMax");
	hsm::svgl::setAttribute(resize, "hsm-arg:parent", this);
	
	view->appendChild(close);
	view->appendChild(resize);
	
	content = new svg::SVGGElement(window->svgDocument);
	content->translate(16, 3);
	view->appendChild(content);
	
	if(parent != 0) {
		parent->insert(this);
	} else {
		window->lock();
		window->svgContent->appendChild(view);
		setGeometry(-.5, -.5, *window/"size"/hsm::Point::X + 1, *window/"size"/hsm::Point::Y + 1);
		window->unlock();
	}
	
	window->instanciateBehaviour(resize);
}

View::~View() {
	(*window/"size").removeReceiver(this);
}

void View::maximize() {
	window->lock();
	resize->setHref("#min");
	float sx, sy, tx, ty;
	hsm::svgl::getGlobalTransform(view, sx, sy, tx, ty, false);
	setGeometry(mx - (tx + .5), my - (ty + .5), *window/"size"/hsm::Point::X + 1, *window/"size"/hsm::Point::Y + 1);
	
	for(View *p = this; p != 0; p = p->parent) {
		p->view->raise();
	}
	window->unlock();
}

void View::minimize() {
	window->lock();
	resize->setHref("#max");
	if(parent != 0) {
		parent->resetGeometry();
	}
	window->unlock();
}

void View::resetGeometry() {
	setGeometry(mx, my, width, height);
}

void View::setGeometry(float x, float y, float w, float h) {
	width = w; height = h;
	mx = x; my = y;
	view->translate(mx, my);
	viewBg->setWidth(width);
	viewBg->setHeight(height);
	name->setX(-(height - 3));
}

void View::process(const hsm::Event &event) {
	window->lock();
	setGeometry(-.5, -.5, *window/"size"/hsm::Point::X + 1, *window/"size"/hsm::Point::Y + 1);
	window->unlock();
}


CompositeView::CompositeView(const std::string &label, hsm::SVGLWindow *w, CompositeView *p /* = 0 */) : View(label, w, p) {
	list = new svg::SVGGElement(window->svgDocument);
	listBg = new svg::SVGRectElement(window->svgDocument);
	listBg->setWidth(120);
	listBg->setFill("white");
	listBg->setStroke("gray");
	list->appendChild(listBg);
	window->lock();
	content->appendChild(list);
	resetGeometry();
	window->unlock();
}

void CompositeView::insert(View *child, unsigned int position /* = 0 */) {
	views.insert(views.begin() + position, child);
	content->appendChild(child->view);
	window->lock();
	resetGeometry();
	window->unlock();
}

void CompositeView::setGeometry(float x, float y, float w, float h) {
	View::setGeometry(x, y, w, h);
	float cHeight = height - 6;
	listBg->setHeight(cHeight);
	unsigned int nChilds = views.size();
	int cWidth = (nChilds > 0 ? (int)((width - 120 - 16) / nChilds) : 0);
	if(cWidth < 16) cWidth = 16;
	int cx = 120;
	int cox;
	for(std::vector< View * >::iterator v = views.begin(); v != views.end(); ++v) {
		cox = cx;
		cx += cWidth;
		if(v + 1 == views.end()) {
			cWidth += ((int)width - cx);
		}
		(*v)->setGeometry(cox, 0, cWidth, cHeight);
	}
}