////////////////////////////////////////////////////////////////////////////////
// File    : ctm.cpp
// Content : collapsable tree map
// History : 11-Jun-2003 - [rb] creation
////////////////////////////////////////////////////////////////////////////////

/*
    Copyright (C) 2004  Renaud Blanch <rb.hsm@laposte.net>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


/* includes *******************************************************************/

#include <cstdlib>
#include <iostream>
#include <sstream>
#include <string>
using namespace std;

#include <svgl/Parser.hpp>

#include <hsm/svgl/SVGLWindow.h>
#include <hsm/system/window/Device.h>
#include <hsm/system/event/EventLoop.h>
#include <hsm/component/Point.h>
#include <hsm/component/ButtonPad.h>

#include "interactions.h"
#include "View.h"


/* handle quit ****************************************************************/

void quitHandler() {
	cout << endl << "bye." << endl;
}


/* main ***********************************************************************/

int main(int argc, const char *argv[]) {
	cout << "ctm - collapsable treemap." << endl << flush;
	atexit(quitHandler);
	
	string path = "/Users/";	
	if(argc >= 2) {
		path = argv[1];
	}
	
	// setting up the window and the pointer
	hsm::SVGLWindow *window = new hsm::impl::Component< hsm::SVGLWindow, hsm::processing::MonoAsync >("collapsable tree map");
	window->show(true);
	
	// svg document
	svgl::Parser parser;
	std::stringstream strstr;
	strstr << "<?xml version='1.0' encoding='iso-8859-1'?>"
		"<!DOCTYPE svg PUBLIC '-//W3C//DTD SVG 03December 1999//EN' 'http://www.w3.org/Graphics/SVG/SVG-19991203.dtd'>"
		"<svg xmlns:hsm='http://hsm.insitu.lri.fr/'"
		"     xmlns:hsm-arg='http://hsm.insitu.lri.fr/'"
		"     xmlns:xlink='http://www.w3.org/1999/xlink'"
		"     xml:space='preserve'"
		"     width='640' height='480'>"
		"	<defs>" 
		"		<linearGradient id='bgd' gradientUnits='userSpaceOnUse' x1='0' y1='0' x2='500' y2='0'>"
		"			<stop offset='0' style='stop-color:lightgray' />"
		"			<stop offset='1' style='stop-color:white' />"
		"		</linearGradient>"

		"		<symbol id='ra'>"
		"			<path d='M6,2 l5,5 l-5,5 z' style='fill:gray; stroke:none' />"
		"		</symbol>"
		"		<symbol id='max'>"
		"			<path  style='stroke:gray; fill:rgb(100,180,100)' d='M-5,2 l5,-5 l5,5z' />"
		"		</symbol>"
		"		<symbol id='min'>"
		"			<path  style='stroke:gray; fill:rgb(100,180,100)' d='M-5,-2 l5,5 l5,-5z' />"
		"		</symbol>"
		"		<symbol id='close'>"
		"			<path style='stroke:gray; fill:rgb(200,100,100)' d='M0,-2 l2,-2 l2,2 l-2,2 l2,2 l-2,2 l-2,-2 l-2,2 l-2,-2 l2,-2 l-2,-2 l2,-2 z'/>"
		"		</symbol>"
		"	</defs>"
		"</svg>";

	svg::SVGDocument *document = parser.parseFromStream(&strstr);
	window->setDocument(document);
	
	// creating instruments
	hsm::Device *pointer = hsm::repository::getDevice("Pointer");
	Clicker::Hsm *clicker = new Clicker::Hsm(Clicker::Data(window, *pointer/"position"/hsm::Point::cast, *pointer/"buttons"/hsm::ButtonPad::L));
	EnterLeaver::Hsm *enterLeaver = new EnterLeaver::Hsm(EnterLeaver::Data(window, *pointer/"position"));
	Translater::Hsm *translater = new Translater::Hsm(Translater::Data(window, *pointer/"buttons"/hsm::ButtonPad::L, *pointer/"position"));
	Zoomer::Hsm *zoomer = new Zoomer::Hsm(Zoomer::Data(window, *pointer/"buttons"/hsm::ButtonPad::R, *pointer/"position"));

	
	CompositeView test(path, window);
	CompositeView t0(path, window, &test);
	View t01(path, window, &t0);
	View t1(path, window, &test);
	
	hsm::eventloop::loop();	


	// clean-up
	delete clicker;
	delete enterLeaver;
	delete translater;
	delete zoomer;
	
	delete window;
	
	return EXIT_SUCCESS;
}