#include <w3c/svg/SVGDocument.hpp>
#include <w3c/svg/SVGElement.hpp>
#include <w3c/svg/SVGCircleElement.hpp>

#include <hsm/interaction/Protocol.h>
#include <hsm/svgl/SVGLBehaviour.h>

#include <hsm/svgl/SVGLWindow.h>
#include <hsm/svgl/Svgl.h>

#include <hsm/system/thread/ThreadLoop.h>
#include <hsm/util/Functor.h>


class Coin : public hsm::SVGLBehaviour {
	static const hsm::SVGLBehaviour::Factory::Register< Coin > registration;
	
	svg::SVGCircleElement *coin;

	hsm::ThreadLoop *animation;
	
	float destY;
	float Y;
	float step;

public :
	Coin(svg::SVGElement *e, hsm::SVGLWindow *w) : hsm::SVGLBehaviour(e, w), step(1) {
		int col, row;
		hsm::svgl::getAttribute(elem, "hsm-arg:col", col);
		hsm::svgl::getAttribute(elem, "hsm-arg:row", row);

		coin = dynamic_cast< svg::SVGCircleElement * >(elem);

		Y = - row * 50.;
		destY = Y + 356;
		coin->setCx(col * 50 + 45);
		coin->setCy(Y);
		coin->setFill("red");
		coin->setStroke("black");
		coin->setR(23);
		coin->lower();
		
		animation = new hsm::ThreadLoop(hsm::Functor::create(this, &Coin::move), 5);
		animation->play();
	}
	
	~Coin() {
		delete animation;
	}
	
	void move() {
		Y += step;
		step += .1;
		if(Y > destY) {
			Y = destY;
			hsm::ThreadLoop::get()->stop();
		}
		window->lock();
		coin->setCy(Y);
		window->unlock();
	}
};

const hsm::SVGLBehaviour::Factory::Register< Coin > Coin::registration("coin");
