#if !defined(TREE_H)
#define TREE_H


/* includes *******************************************************************/

#include "TreeOpener.h"


/* part from original file ****************************************************/

#include <w3c/svg/SVGElement.hpp>
#include <w3c/svg/SVGGElement.hpp>
#include <w3c/svg/SVGUseElement.hpp>
#include <w3c/svg/SVGRectElement.hpp>

#include <hsm/interaction/Protocol.h>
#include <hsm/svgl/SVGLWindow.h>
#include <hsm/svgl/SVGLBehaviour.h>


/* foraward declarations ******************************************************/

class Tree;


/* Node class definition ******************************************************/

class Node : public hsm::SVGLBehaviour {
	friend class Tree;
	
protected :
	svg::SVGRectElement *background;
	float height;
	Tree *parent;

private :
	static const hsm::SVGLBehaviour::Factory::Register< Node > registration;


public :
	Node(svg::SVGElement *e, hsm::SVGLWindow *w);
	virtual ~Node();
	
protected :
	virtual float getHeight();

	void translateAllBrothers(float dy);
	void translateBrothers(float dy);
};


/* Tree class definition ******************************************************/

class TreeOpener;

class Tree : public Node {
	friend class TreeOpener;
	friend class Node;

public : 
	bool isOpen;

private :
	svg::SVGUseElement *handle;
	svg::SVGGElement *content;
	
	static const hsm::SVGLBehaviour::Factory::Register< Tree > registration;
	
public :
	Tree(svg::SVGElement *e, hsm::SVGLWindow *w);

	void open();
	void close();

	void add(svg::SVGElement *e, unsigned int p);
	void remove(svg::SVGElement *e);

protected :
	virtual float getHeight();
	void resizeBackground(float dHeight);
};


#endif // !defined(TREE_H)
