////////////////////////////////////////////////////////////////////////////////
// File    : sp.cpp
// Content : a simple program to test semantic pointing
// History : 11-Jul-2005 - [rb] creation
////////////////////////////////////////////////////////////////////////////////

/*
 Copyright (C) 2005  Renaud Blanch <rb.hsm@laposte.net>
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.
 
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


/* includes *******************************************************************/

#include <cstdlib>
#include <iostream>
using namespace std;

#include <hsm/svgl/SVGLWindow.h>
#include <hsm/system/window/Device.h>
#include <hsm/system/event/EventLoop.h>
#include <hsm/component/ButtonPad.h>


#include <interactions.h>


/* handle quit ****************************************************************/

void quitHandler() {
	cout << endl << "bye." << endl;
}


/* main ***********************************************************************/

int main(int argc, const char *argv[]) {
	cout << "sp - a simple program to test semantic pointing." << endl << flush;
	atexit(quitHandler);

	// setting up the window and the pointer
	hsm::SVGLWindow *window = new hsm::impl::Component< hsm::SVGLWindow, hsm::processing::MonoAsync >("semantic pointing");
	if(argc != 2) {
		cerr << "usage : sp file.svg" << endl;
		return EXIT_FAILURE;
	}
	window->setDocument(argv[1]);

	hsm::Device *pointer = hsm::repository::getDevice("Pointer");

	// creating instrument
	Translater::Hsm *translater = new Translater::Hsm(Translater::Data(window, *pointer/"buttons"/hsm::ButtonPad::L, *pointer/"position"));
	Scaler::Hsm *scaler = new Scaler::Hsm(Scaler::Data(window, *pointer/"position"));


	// launching interaction
	window->show(true);
	hsm::eventloop::loop();	
	
	
	// clean-up
	delete scaler;
	delete translater;
	delete window;
	
	
	return EXIT_SUCCESS;
}
