////////////////////////////////////////////////////////////////////////////////
// File    : 01-simple.cpp
// Content : a simple program to test interactions
// History : 09-Jul-2004 - [rb] creation
////////////////////////////////////////////////////////////////////////////////

/*
 Copyright (C) 2004  Renaud Blanch <rb.hsm@laposte.net>
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.
 
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


/* includes *******************************************************************/

#include <cstdlib>
#include <iostream>
#include <sstream>
using namespace std;

#include <hsm/svgl/SVGLWindow.h>
#include <hsm/system/window/Device.h>
#include <hsm/system/event/EventLoop.h>
#include <hsm/component/ButtonPad.h>

#include <svgl/Parser.hpp>

#include <interactions.h>


/* handle quit ****************************************************************/

void quitHandler() {
	cout << endl << "bye." << endl;
}


/* main ***********************************************************************/

int main(int argc, const char *argv[]) {
	cout << "01-simple - a simple program to test simple interactions." << endl << flush;
	atexit(quitHandler);

	// setting up the window and the pointer
	hsm::SVGLWindow *window = new hsm::impl::Component< hsm::SVGLWindow, hsm::processing::MonoAsync >("simple");
	hsm::Device *pointer = hsm::repository::getDevice("Pointer");

	// creating instrument
	Translater::Hsm *translater = new Translater::Hsm(Translater::Data(window, *pointer/"buttons"/hsm::ButtonPad::L, *pointer/"position"));
	Zoomer::Hsm *zoomer = new Zoomer::Hsm(Zoomer::Data(window, *pointer/"buttons"/hsm::ButtonPad::R, *pointer/"position"));
	
	
	// creating the svg document
	svgl::Parser parser;
	std::stringstream strstr;
	strstr << 
		"<?xml version='1.0' encoding='iso-8859-1'?>"
		"<!DOCTYPE svg PUBLIC '-//W3C//DTD SVG 1.0//EN' 'http://www.w3.org/TR/2001/REC-SVG-20010904/DTD/svg10.dtd'>"
		"<svg xmlns:hsm='http://insitu.lri.fr/~blanch/projects/hsm/' width='300' height='200'>"
		"	<g hsm:behaviour='visual/PanAndZoomable'>"
		"		<text x='10' y='40' style='font-size:30; fill:black'>pan and zoom</text>"
		"		<rect x='10' y='23' height='18' width='193' style='stroke:none; fill:black; fill-opacity:0' />"
		"	</g>"
		"</svg>";
		
	svg::SVGDocument *document = parser.parseFromStream(&strstr);
	window->setDocument(document);


	// launching interaction
	window->show(true);
	hsm::eventloop::loop();	
	
	
	// clean-up
	delete translater;
	delete zoomer;
	delete window;
	
	
	return EXIT_SUCCESS;
}
