////////////////////////////////////////////////////////////////////////////////
// File    : Tree.h
// Content : a tree implementation
// History : 09-Jun-2005 - [rb] (re)creation
////////////////////////////////////////////////////////////////////////////////

/*
    Copyright (C) 2004  Renaud Blanch <rb.hsm@laposte.net>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#if !defined(TREE_H)
#define TREE_H


/* includes *******************************************************************/

#include "TreeOpener.h"

#include <hsm/interaction/Protocol.h>
#include <hsm/svgl/SVGLWindow.h>
#include <hsm/svgl/SVGLBehaviour.h>


/* forward declarations *******************************************************/

class Tree;

namespace svg {
	class SVGElement;
	class SVGGElement;
	class SVGUseElement;
	class SVGRectElement;
}


/* Node class definition ******************************************************/

class Node : public hsm::SVGLBehaviour {
	friend class Tree;
	
protected :
	svg::SVGRectElement *background;
	float height;
	Tree *parent;

private :
	static const hsm::SVGLBehaviour::Factory::Register< Node > registration;


public :
	Node(svg::SVGElement *e, hsm::SVGLWindow *w);
	
protected :
	virtual float getHeight();

	void translateAllBrothers(float dy);
	void translateBrothers(float dy);
};


/* Tree class definition ******************************************************/

class TreeOpener;

class Tree : public Node {
	friend class TreeOpener;
	friend class Node;

public : 
	bool isOpen;

private :
	svg::SVGUseElement *handle;
	svg::SVGGElement *content;
	
	static const hsm::SVGLBehaviour::Factory::Register< Tree > registration;
	
public :
	Tree(svg::SVGElement *e, hsm::SVGLWindow *w);

	void open();
	void close();

protected :
	virtual float getHeight();
	void resizeBackground(float dHeight);
};


#endif // !defined(TREE_H)
