#include <w3c/svg/SVGElement.hpp>
#include <w3c/svg/SVGGElement.hpp>

#include <hsm/interaction/Protocol.h>
#include <hsm/svgl/SVGLWindow.h>
#include <hsm/svgl/SVGLBehaviour.h>

#include <hsm/util/Debug.h>
#include <hsm/svgl/Svgl.h>


class Menu : public hsm::SVGLProtocol, public hsm::EnterLeave, public hsm::PushPop {
	Menu *parent;
	Menu *current;

	svg::SVGGElement *svgLabel;
	svg::SVGGElement *svgSubmenu;
	
	static const hsm::SVGLBehaviour::Register< Menu > registration;
	
public :
	Menu(svg::SVGElement *e, hsm::SVGLWindow *w) : hsm::SVGLProtocol(e, w) {
		hsm::svgl::getAttribute(e, "hsm-arg:parent", parent);
		
		HSM_OUTPUT("Menu : " << this << ", " << parent)
		
		hsm::svgl::Childs(elem).get(svgLabel)
									  .get(svgSubmenu);
		
		if(svgSubmenu != 0) {
			dom::Node *child = svgSubmenu->getFirstChild();
			svg::SVGGElement *sub;
			while((sub = hsm::svgl::getTypedBrother< svg::SVGGElement * >(child)) != 0) {
				hsm::svgl::setAttribute(sub, "hsm-arg:parent", this);
			}

			window->instanciateBehaviour(svgSubmenu);
			elem->removeChild(svgSubmenu);
		}
	}
	
	void setCurrent(Menu *c) {
		if(current != 0) {
			current->close();
		}
		current = c;
	}
	
	void open() {
		window->lock();
		if(svgSubmenu != 0) {
			elem->appendChild(svgSubmenu);
		}
		svg::SVGStylable *styl = dynamic_cast< svg::SVGStylable * >(elem);
		styl->setFill("lightblue");
		window->unlock();
	}
	
	void close() {
		window->lock();
		if(svgSubmenu != 0) {
			elem->removeChild(svgSubmenu);
		}
		svg::SVGStylable *styl = dynamic_cast< svg::SVGStylable * >(elem);
		styl->setFill("white");
		window->unlock();	
	}
	
	virtual void enter() {
		if(parent != 0) {
			parent->setCurrent(this);
		}
		open();
	}
	
};

const hsm::SVGLBehaviour::Register< Menu > Menu::registration("menu");
