////////////////////////////////////////////////////////////////////////////////
// File    : Value.h
// Content : Value component
// History : 26-Nov-2001 - [rb] creation
//           28-Feb-2002 - [rb] added assignement operator for Point class
//           31-Jan-2003 - [rb] removed automatic dispatching from Values to
//                              Point and threshold for Point
//			    17-Feb-2004 - [rb] inlined most of the code
////////////////////////////////////////////////////////////////////////////////

/*
    Copyright (C) 2004  Renaud Blanch <rb.hsm@laposte.net>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


// ToDo ////////////////////////////////////////////////////////////////////////
// - add caracteristics to the Values (bounds, ...)


#if !defined(HSM_VALUE_H)
#define HSM_VALUE_H


/* includes *******************************************************************/

#include <hsm/core/Component.h>


namespace hsm {

/* Value class definition *****************************************************/

	class Value : public impl::Component< > {
	public :
		static const Value *cast;
		
	protected :
		float value;

	public :
		Value(const Id &id, float v = 0);
		Value(float v = 0);

		inline void set(float v) { value = v; }
		inline void add(float f) { value += f; }
		inline void sub(float f) { value -= f; }
		inline void mul(float f) { value *= f; }
		inline void div(float f) { value /= f; }
		
		inline operator const float() const { return value; }

		inline Value &operator=(const Value &v) { return operator=(float(v)); }
		
#define OPERATOR(op, method)                                \
		inline Value &operator op (float f) {						\
			float old = value;                                 \
			method (f);                                        \
			dispatch(Event(this, Event::CHANGE, value - old)); \
		   return *this;                                      \
		}

		OPERATOR(=,  set)
		OPERATOR(+=, add)
		OPERATOR(-=, sub)
		OPERATOR(*=, mul)
		OPERATOR(/=, div)
		
#undef OPERATOR

	};

	
} // namespace hsm


#endif // !defined(HSM_VALUE_H)
