////////////////////////////////////////////////////////////////////////////////
// File    : Composite.cpp
// Content : a common interface class for hsm composite component
// History : 10-Jan-2004 - [rb] creation
////////////////////////////////////////////////////////////////////////////////

/*
    Copyright (C) 2004  Renaud Blanch <rb.hsm@laposte.net>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


/* includes *******************************************************************/

#include <Composite.h>


namespace hsm {

/* Composite class implementation *********************************************/

	const Composite *Composite::cast = 0;
	
	Composite::Composite(const Id &id /* = Id() */) : Component(id) {}

	Composite::~Composite() {
		for(Map::iterator i = components.begin(); i != components.end(); ++i) {
			delete (*i).second;
		}
	}

	void Composite::add(Component *component) {
		components[component->getId()] = component;
		dispatch(Event(this, Event::NEW, component));
		component->addReceiver(this);
	}

	void Composite::remove(Component *component) {
		component->removeReceiver(this);
		components.erase(component->getId());
	}

	Component *Composite::get(const Id &id) const {
		Component *res = 0;
		Map::const_iterator i = components.find(id);
		if(i != components.end()) {
			res = (*i).second;
		}
		return res;
	}

	Component &operator/(Component &component, const Component::Id &id) {
		Composite *c = dynamic_cast< Composite * >(&component);
		if(c != 0) {
			return *(c->components[id]);
		} else {
			return component;
		}
	}

	Component &operator/(Component &component, const char *id) {
		return operator/(component, Component::Id(id));
	}


} // namespace hsm
