////////////////////////////////////////////////////////////////////////////////
// File    : PlugIn.cpp
// Content : functions dealing with dynamic library loading
// History : 11-Jan-2004 - [rb] creation
////////////////////////////////////////////////////////////////////////////////

/*
    Copyright (C) 2004  Renaud Blanch <rb.hsm@laposte.net>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


// ToDo(-) / Done(+) ///////////////////////////////////////////////////////////
// - getenv the plugin path on WIN32


/* includes *******************************************************************/

#include <PlugIn.h>

#include <hsm/util/Debug.h>

#if defined(MACOSX)
#  include <mach-o/dyld.h>
#elif defined(WIN32)
#  define _WIN32_WINNT 0x0400
#  include <windows.h>
#endif

#include <string>
#include <cstdlib>


namespace hsm {

	namespace plugin {

/* functions ******************************************************************/

		bool load(const std::string &name, std::string path /* = "" */) {			
			if(path == "") {
				// looking env to find plugins path 
				std::string pathes;
				char *cdl = std::getenv("HSM_PLUGINS_PATH");
				if(cdl != 0) {
					pathes = cdl;
				}
				
				for(int i = pathes.find(":"); i >= 0; i = pathes.find(":")) {
					if(i == 0) continue;
					
					path = pathes.substr(0, i);
					pathes = pathes.substr(i + 1);

					if(load(name, path)) {
						return true;
					}
				}
				path = pathes;
				if(path != "") {
					return load(name, path);
				}
			}
			
			// looking into path
			bool res = true;

			std::string prefix;
			prefix = name.substr(0, name.find("."));
				
#if defined(MACOSX) ////////////////////////////////////////////////////////////
			path += "/" + prefix + ".bundle";

			NSObjectFileImage image;
			if(!(res = (NSCreateObjectFileImageFromFile(path.c_str(), &image) == NSObjectFileImageSuccess))) {
				HSM_ERRPUT("Unable to load the component : " + prefix + " (NSCreateObjectFileImageFromFile() failed)");
			}
			if(res) {
				NSModule module = NSLinkModule(image, path.c_str(), NSLINKMODULE_OPTION_PRIVATE | NSLINKMODULE_OPTION_BINDNOW);
				NSDestroyObjectFileImage(image) ;
				if(!(res = (module != 0))) {
					HSM_ERRPUT("Unable to load the component : " + prefix + " (NSLinkModule() failed)");
				}
			}
			
#elif defined(WIN32) ///////////////////////////////////////////////////////////
			HMODULE comp = LoadLibrary(prefix.c_str());
			if(!(res = (comp == 0))) {
				HSM_ERRPUT("Unable to load the component : " + prefix);
			}

#endif /////////////////////////////////////////////////////////////////////////
			return res;
		}


	} // namespace plugin

} // namespace hsm
