////////////////////////////////////////////////////////////////////////////////
// File    : Mutex.h
// Content : a Mutex classe
// History :
////////////////////////////////////////////////////////////////////////////////

/*
    Copyright (C) 2004  Renaud Blanch <rb.hsm@laposte.net>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


/// \file Mutex.h


#if !defined(HSM_MUTEX_H)
#define HSM_MUTEX_H


/* includes *******************************************************************/

#if !defined(WIN32)
#  include <pthread.h>
#  include <unistd.h>
#else
#  define _WIN32_WINNT 0x0400
#  include <windows.h>
#endif


namespace hsm {

/* Mutex class declaration ****************************************************/

	/// Implements a mutex abstraction.
	class Mutex {

		// os dependent declaration
		struct os {
			// typedefs
		#if !defined(WIN32)
			typedef pthread_mutex_t  *Id; ///< mutex descriptor
		#else
			typedef CRITICAL_SECTION *Id; ///< mutex descriptor
		#endif

			// functions
			static Id   create();
			static void lock(Id &id);
			static bool trylock(Id &id);
			static void unlock(Id &id);
		};

		friend struct os;

	private :
		os::Id mutex;

	public :
		         Mutex() { mutex = os::create(); } ///< constructor
		virtual ~Mutex() { delete mutex; }         ///< destructor
	
		inline void lock()    { os::lock(mutex); }   ///< locks the mutex
		inline bool trylock() { return os::trylock(mutex); } ///< tries to lock the mutex
		inline void unlock()  { os::unlock(mutex); } ///< unlocks the mutex
	};


} // namespace hsm


#endif // !defined(HSM_MUTEX_H)
