////////////////////////////////////////////////////////////////////////////////
// File    : Timer.cpp
// Content : a timer class
// History : 21-Dec-2001 - [rb] creation
////////////////////////////////////////////////////////////////////////////////

/*
    Copyright (C) 2004  Renaud Blanch <rb.hsm@laposte.net>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


/* includes *******************************************************************/

#include <Timer.h>

#include <hsm/util/Functor.h>
#include <hsm/util/Debug.h>


namespace hsm {

/* static data initialisation *************************************************/

	Timer::Map Timer::timerMap;


/* Timer class implementation *************************************************/

	Timer::Timer(int ms /* = 0 */, Functor *tfn /* = 0 */, void *d /* = 0 */)
	 : delay(ms), timerFn(tfn), data(d), fired(false), enabled(true) {
		if(timerFn == 0) timerFn = new Functor;
		install();
	}

	Timer::~Timer() {
		remove();
	}

	void Timer::stop() {
		enabled = false;
	}

	void Timer::rearm(int ms /* = 0 */) {
		remove();
		if(ms != 0) { delay = ms; }
		fired = false;
		enabled = true;
		install();
	}

	bool Timer::hasFired() const {
		return fired;
	}

	void Timer::fire() {
		if(enabled) {
			enabled = false;
			(*timerFn)(data);
			fired = true;
		}
	}

	void Timer::install() {
		if(delay >= 0) {
			timerId = os::installTimer(delay);
			timerMap[timerId] = this;
		}
	}

	void Timer::remove() {
		if(delay >= 0) {
			os::removeTimer(timerId);
			timerMap.erase(timerId);
		}
	}


} // namespace InputLibrary
