////////////////////////////////////////////////////////////////////////////////
// File    : GLWindow.cpp
// Content : Window class with openGL support
// History : 11-Oct-2001 - [rb] creation
////////////////////////////////////////////////////////////////////////////////

/*
    Copyright (C) 2004  Renaud Blanch <rb.hsm@laposte.net>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


/* includes *******************************************************************/

#include <GLWindow.h>

#include <hsm/component/Point.h>
#include <hsm/util/Debug.h>


namespace hsm {

/* GLWindow class implementation **********************************************/

	GLWindow::GLWindow(const Id &id /* = Id() */) : Window(id) {
		os::createGlContext(this);
		os::makeCurrent(this); 
	}

	GLWindow::~GLWindow() {
		os::destroyGlContext(this);
	}

	void GLWindow::show(bool s /* = true */) {
		Window::show(s);
		if(s) {
			glMutex.lock();
			doInit();
			doReshape();
			doDisplay();
			glMutex.unlock();
		}
	}
	
	void GLWindow::setVBLSync(long vbl /* = 1 */) {
		os::setVBLSync(this, vbl);
	}

	void GLWindow::setFullScreen(bool full /* = true */) {
		os::setFullScreen(this, full);
	}


	void GLWindow::init() {
		glClearColor(1.0, 1.0, 1.0, 1.0);
		glEnable(GL_DEPTH_TEST);
		
		glEnable(GL_NORMALIZE);
		glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);
		
		glEnable(GL_BLEND);
		glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
	}

	void GLWindow::display() {
		glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);	
	}
	
	void GLWindow::reshape(unsigned int width, unsigned int height) {
		glViewport(0, 0, width, height);
		glMatrixMode(GL_PROJECTION);
		glLoadIdentity();
		glOrtho(0., width, height, 0., -1., 1.);
		glMatrixMode(GL_MODELVIEW);
		glLoadIdentity();
	}
	
	void GLWindow::process(const Event &event) {
		glMutex.lock();
		if(static_cast< void * >(event.sender) == static_cast< void * >(size)) {
			doReshape();
		}
		doDisplay();
		glMutex.unlock();
	}
	
	void GLWindow::doInit() {
		os::makeCurrent(this);
		init();
	}
	
	void GLWindow::doReshape() {
		os::makeCurrent(this);
		os::reshape(this);
		reshape(*size/Point::X, *size/Point::Y);
	}

	void GLWindow::doDisplay() {
		os::makeCurrent(this);
		display();
		os::swapBuffers(this);
	}


} // namespace hsm
