////////////////////////////////////////////////////////////////////////////////
// File    : Keyboard.MACOSX.cpp
// Content : libhsm system keyboard plugin
// History : 13-Aug-2002 - [rb] creation
////////////////////////////////////////////////////////////////////////////////

/*
    Copyright (C) 2004  Renaud Blanch <rb.hsm@laposte.net>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


// ToDo ////////////////////////////////////////////////////////////////////////
// - switch from ascii code to low level key codes ?
//   (and add non ascii keys)


/* includes *******************************************************************/

#include <Keyboard.h>

#include <hsm/util/Debug.h>
#include <hsm/component/ButtonPad.h>
#include <hsm/system/window/Window.h>


namespace hsm {

/* Keyboard instance **********************************************************/

	const Keyboard Keyboard::instance;


/* Keyboard implementation ****************************************************/

	OSStatus Keyboard::handler(EventHandlerCallRef inRef, EventRef inEvent, void *userData) {
		unsigned char keyMacCharCodes;
		unsigned int keyModifiers;

		switch(GetEventClass(inEvent)){
		case kEventClassKeyboard :
			switch(GetEventKind(inEvent)){
			case kEventRawKeyDown :
				GetEventParameter(inEvent, kEventParamKeyMacCharCodes, typeChar, 0, sizeof(char), 0, &keyMacCharCodes);
				*(instance.keys)/ButtonPad::ButtonId(keyMacCharCodes) = true;
				break;

			case kEventRawKeyRepeat :
				GetEventParameter(inEvent, kEventParamKeyMacCharCodes, typeChar, 0, sizeof(char), 0, &keyMacCharCodes);
				switch(keyMacCharCodes) {
				// just repeating arrow keys & backspace
				case ButtonPad::LEFT :
				case ButtonPad::RIGHT :
				case ButtonPad::BACKSPACE :
					*(instance.keys)/ButtonPad::ButtonId(keyMacCharCodes) = false;
					*(instance.keys)/ButtonPad::ButtonId(keyMacCharCodes) = true;
					break;
				}
				break;

			case kEventRawKeyUp :
				GetEventParameter(inEvent, kEventParamKeyMacCharCodes, typeChar, 0, sizeof(char), 0, &keyMacCharCodes);
				*(instance.keys)/ButtonPad::ButtonId(keyMacCharCodes) = false;
				break;
				
			case kEventRawKeyModifiersChanged :
				GetEventParameter(inEvent, kEventParamKeyModifiers, typeUInt32, 0, sizeof(unsigned int), 0, &keyModifiers);
				*(instance.modifiers)/ButtonPad::SHIFT = (keyModifiers & shiftKey);
				*(instance.modifiers)/ButtonPad::ALT   = (keyModifiers & optionKey);
				*(instance.modifiers)/ButtonPad::CTRL  = (keyModifiers & controlKey);
				break;
			}
			break;
		}
		return eventNotHandledErr;
	}


	Keyboard::Keyboard() : impl::Component< Device >("Keyboard") {
		HSM_PRINT(1, "[Keyboard::Keyboard()]");

		nbEvents = 4;
		eventSpec = new Window::os::EventSpec[nbEvents];
		eventSpec[0].eventClass = kEventClassKeyboard;
		eventSpec[0].eventKind = kEventRawKeyDown;
		eventSpec[1].eventClass = kEventClassKeyboard;
		eventSpec[1].eventKind = kEventRawKeyUp;
		eventSpec[2].eventClass = kEventClassKeyboard;
		eventSpec[2].eventKind = kEventRawKeyRepeat;
		eventSpec[3].eventClass = kEventClassKeyboard;
		eventSpec[3].eventKind = kEventRawKeyModifiersChanged;

		add(keys = new ButtonPad("keys", ButtonPad::MAX + 1));
		add(modifiers = new ButtonPad("modifiers", ButtonPad::CTRL + 1));
		
		hsm::repository::add(this);
		registerDevice();
	}

	Keyboard::~Keyboard() {
		HSM_PRINT(1, "[Keyboard::~Keyboard()]");

		hsm::repository::remove(this);
		delete[] eventSpec;
	}

	Window::os::Handler *Keyboard::getHandler() const {
		return handler;
	}


} // namespace hsm
