////////////////////////////////////////////////////////////////////////////////
// File    : test-Component.cpp
// Content : a test for hsm::Component
// History : 08-Jan-2004 - [rb] creation
////////////////////////////////////////////////////////////////////////////////

/*
    Copyright (C) 2004  Renaud Blanch <rb.hsm@laposte.net>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


/* includes *******************************************************************/

#include <cstdlib>
#include <iostream>
#include <string>
using namespace std;

#include <hsm/core/Component.h>
#include <hsm/system/thread/Thread.h>


/* globals ********************************************************************/

class OutComponent : public hsm::impl::Component< > {
protected :
	void process(const hsm::Event &event) {
		cout << "[Sync received " << event << "]" << endl;
	}
};

class AsyncOutComponent : public hsm::impl::Component< hsm::Component, hsm::processing::Async > {
protected :
	void process(const hsm::Event &event) {
		cout << "[Async received " << event << "]" << endl;
	}
};

class ForwardComponent : public hsm::impl::Component< > {
protected :
	void process(const hsm::Event &event) {
		cout << "dispatching " << event << endl;
		dispatch(event);
	}
};


typedef hsm::impl::Component< hsm::Component, hsm::processing::Sync, hsm::processing::Functor > FunctorComponent;

void outFn(void *d) {
	cout << "[Fn received " << *reinterpret_cast< char * >(d) << "]" << endl;
}


/* handle quit ****************************************************************/

void quitHandler() {
	cout << endl <<"by." << endl;
}


/* main ***********************************************************************/

int main(int argc, const char *argv[]) {
	cout << "test-Component - a program to test the hsm::Component class." << endl;
	atexit(quitHandler);

	hsm::Component *out = new OutComponent;
	hsm::Component *aout = new AsyncOutComponent;
	hsm::Component *forward = new ForwardComponent;

	hsm::Component *fout = new FunctorComponent(hsm::Functor::create(outFn));

	forward->addReceiver(out);
	forward->addReceiver(aout);
	forward->addReceiver(fout);
	
	char a = 'a';
	forward->post(hsm::Event(forward, 0, &a));
	
	forward->removeReceiver(out);
	forward->removeReceiver(aout);
	forward->removeReceiver(fout);

	hsm::thread::sleep(1000);

	delete forward;
	delete out;
	delete aout;
	delete fout;

	return EXIT_SUCCESS;
}
