////////////////////////////////////////////////////////////////////////////////
// File    : test-Mutex.cpp
// Content : a test for hsm Mutex
// History : 25-Sep-2002 - [rb] creation
////////////////////////////////////////////////////////////////////////////////

/*
    Copyright (C) 2003  Renaud Blanch <rb.hsm@laposte.net>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


/* includes *******************************************************************/

#include <cstdlib>
#include <iostream>
#include <string>
using namespace std;

#include <hsm/system/thread/Mutex.h>
#include <hsm/system/thread/ThreadLoop.h>
#include <hsm/system/thread/Thread.h>
#include <hsm/util/Functor.h>


/* globals ********************************************************************/

hsm::Mutex *mutex = 0;

void loop(void *c) {
	mutex->lock();
	for(unsigned int i = 0; i < 10; i++) {
		cout << *reinterpret_cast< char * >(c) << flush;
		hsm::thread::sleep(250);
	}
	mutex->unlock();
}
	

/* handle quit ****************************************************************/

void quitHandler() {
	cout << endl <<"bye." << endl;
}


/* main ***********************************************************************/

int main(int argc, const char *argv[]) {
	cout << "test-Mutex - a program to test the hsm::Mutex class." << endl;
	atexit(quitHandler);

	char a = 'a';
	char b = 'b';

	mutex = new hsm::Mutex();
	
	hsm::Functor	*l = hsm::Functor::create(loop);
	hsm::ThreadLoop *t1 = new hsm::ThreadLoop(l, 10, reinterpret_cast< void * >(&a));
	hsm::ThreadLoop *t2 = new hsm::ThreadLoop(l, 10, reinterpret_cast< void * >(&b));

	t1->play();
	t2->play();

	hsm::thread::sleep(10000);
	
	t1->stop();
	t2->stop();
	
	hsm::ThreadLoop::waitFor(t1);
	hsm::ThreadLoop::waitFor(t2);
	
	delete t1;
	delete t2;
	
	delete l;

	delete mutex;

	return EXIT_SUCCESS;
}
